#ifndef GENERATOR_HPP
#define GENERATOR_HPP

#include <unordered_map>
#include <string>

void InitializeGenerator(void);
int GenerateRandomNumber(int generationLimit);

struct ArgumentList {
    std::string filename = "";
    bool isFileSet = false;
    int prefixLength = 0;
    bool isPrefixSet = false;
    int outputLength = 0;
    bool isOutputSet = false;
};

struct TrieNode {
public:
    TrieNode();
    std::unordered_map<char, TrieNode*> children;
    int occurrences = 1;
    double probability = 0.;
    bool isEndOfWord;
};

struct Trie {
public:
    Trie();
    void Insert(std::string kgram);
    void Recalculate(std::unordered_map<char, TrieNode*> children);
    char GetNextCharacter(const std::string& kgram);
    TrieNode* root;
};


struct Generator {
public:
    Generator(void) = default;
    ~Generator(void) = default;
    void SetArguments(const int argc, char* argv[]);
    void Train(void);
    void Generation(void);
private:
    std::string currentKGram;
    ArgumentList setup;
    Trie trie;
};

void PrintUsage(void);

#endif // !GENERATOR_HPP
